// mmcQueue.cpp: implementation of the mmcQueue class.
//
//////////////////////////////////////////////////////////////////////

#include "mmcQueue.h"
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "support.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

mmcQueue::mmcQueue()
{
	ArrivalRate = (double) 0;
	ServiceRate = (double) 0;
	NumServers = 0;
	TimeInSystem = (double) 0;
	TimeInQueue = (double) 0;
	NumInQueue = (double) 0;
	NumInSystem = (double) 0;
	p0 = (double) 0;
	Stable = true;
}

mmcQueue::mmcQueue(char *name, double lambda, double mu, int c) {
	ID = strdup(name);
	ArrivalRate = lambda;
	ServiceRate = mu;
	NumServers = c;
	Utilization = ArrivalRate / ((double) NumServers * ServiceRate);
	if (Utilization < 1) {
		Stable = true;
		calc();
	} else {
		Stable = false;
		TimeInSystem = (double) 0;
		TimeInQueue = (double) 0;
		NumInQueue = (double) 0;
		NumInSystem = (double) 0;
		p0 = (double) 0;
	}		
}


mmcQueue::~mmcQueue()
{

}

void mmcQueue::show()
{
	printf("\nQueue:\t%s", ID);
	printf("\nArrival:\t%8.4f", ArrivalRate);
	printf("\nService:\t%8.4f", ServiceRate);
	printf("\nServers:\t%8d", NumServers);
	printf("\nUtilization:\t%8.5f", Utilization);
	if (isStable()) printf(" (system stable)");
	else printf (" (system unstable)");
	printf("\np(0):\t\t%8.5f", p0);
	printf("\nNumInSystem:\t%8.4f", NumInSystem);
	printf("\nNumInQueue:\t%8.4f", NumInQueue);
	printf("\nTimeInSystem:\t%8.4f", TimeInSystem);
	printf("\nTimeInQueue:\t%8.4f", TimeInQueue);
	printf("\n\n");
}


void mmcQueue::calc()
{
	double tmp;

	if (Utilization > 0 && Utilization < 1) {
		if (NumServers == 1) {
			p0 = (double) 1 - Utilization;
			NumInQueue = pow(Utilization, (double) 2)/((double) 1 - Utilization);
			NumInSystem = Utilization / ((double) 1 - Utilization);
			TimeInSystem = (double) 1 / (ServiceRate * ((double) 1-Utilization));
			TimeInQueue = TimeInSystem*Utilization;
		} else {
			p0 = pow(((double) NumServers*Utilization), (double) NumServers);
			p0 = p0/((double) factorial(NumServers) * ((double) 1-Utilization));
			p0 = p0+(double) 1;
			for (int i = 1; i < NumServers; i++) {
				tmp = pow(((double) NumServers*Utilization), (double) i);
				tmp = tmp / ((double) factorial(i));
				p0 = p0 + tmp;
			}
			p0 = pow(p0, (double) (-1));
			NumInQueue = Utilization * pow((double) NumServers*Utilization, (double) NumServers) * p0;
			NumInQueue = NumInQueue / ((double) factorial(NumServers) * ((double) 1-Utilization) * ((double) 1-Utilization));
			NumInSystem = NumInQueue + (ArrivalRate/ServiceRate);
			TimeInQueue = NumInQueue / ArrivalRate;
			TimeInSystem = NumInSystem / ArrivalRate;
		}
	}
}


